@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -35,11 +35,13 @@ import java.security.PrivilegedExceptionAction;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Date;
+import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Properties;
 import java.util.Queue;
 import java.util.concurrent.ConcurrentLinkedQueue;
+import java.util.concurrent.atomic.AtomicReference;
 import javax.mail.Address;
 import javax.mail.Message.RecipientType;
 import javax.mail.MessagingException;
@@ -75,6 +77,7 @@ import com.openexchange.mail.MailPath;
 import com.openexchange.mail.api.AuthType;
 import com.openexchange.mail.api.AuthenticationFailedHandler.Service;
 import com.openexchange.mail.api.AuthenticationFailedHandlerService;
+import com.openexchange.mail.api.AuthenticationFailedStateUtility;
 import com.openexchange.mail.api.AuthenticationFailureHandlerResult;
 import com.openexchange.mail.api.AuthenticationFailureHandlerResult.Type;
 import com.openexchange.mail.api.MailAccess;
@@ -101,9 +104,13 @@ import com.openexchange.mail.transport.listener.Reply;
 import com.openexchange.mail.transport.listener.Result;
 import com.openexchange.mail.usersetting.UserSettingMail;
 import com.openexchange.mailaccount.Account;
+import com.openexchange.mailaccount.Constants;
 import com.openexchange.mailaccount.MailAccount;
 import com.openexchange.mailaccount.MailAccountStorageService;
+import com.openexchange.mailaccount.MailAccounts;
 import com.openexchange.mailaccount.TransportAuth;
+import com.openexchange.mailaccount.utils.MailAccountUtils;
+import com.openexchange.net.HostList;
 import com.openexchange.net.ssl.SSLSocketFactoryProvider;
 import com.openexchange.net.ssl.config.SSLConfigurationService;
 import com.openexchange.net.ssl.exception.SSLExceptionCode;
@@ -141,16 +148,16 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
      */
     protected static final String SMTP = SMTPProvider.PROTOCOL_SMTP.getName();
 
-    private static volatile String staticHostName;
+    private static final AtomicReference<String> STATIC_HOST_NAME_REF = new AtomicReference<>();
 
-    private static volatile UnknownHostException warnSpam;
+    private static final AtomicReference<UnknownHostException> WARN_SPAM_REF = new AtomicReference<>();
 
     static {
         try {
-            staticHostName = InetAddress.getLocalHost().getCanonicalHostName();
+            STATIC_HOST_NAME_REF.set(InetAddress.getLocalHost().getCanonicalHostName());
         } catch (UnknownHostException e) {
-            staticHostName = "localhost";
-            warnSpam = e;
+            STATIC_HOST_NAME_REF.set("localhost");
+            WARN_SPAM_REF.set(e);
         }
     }
 
@@ -166,8 +173,8 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
     protected final Session session;
 
     private final Queue<Runnable> pendingInvocations;
-    private volatile javax.mail.Session smtpSession;
-    private volatile SMTPConfig cachedSmtpConfig;
+    private final AtomicReference<javax.mail.Session> smtpSessionRef;
+    private final AtomicReference<SMTPConfig> cachedSmtpConfigRef;
     private User user;
     private transient Subject kerberosSubject;
 
@@ -176,10 +183,12 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
      */
     protected AbstractSMTPTransport() {
         super();
-        accountId = MailAccount.DEFAULT_ID;
+        smtpSessionRef = new AtomicReference<>();
+        cachedSmtpConfigRef = new AtomicReference<>();
+        accountId = Account.DEFAULT_ID;
         ctx = null;
         session = null;
-        pendingInvocations = new ConcurrentLinkedQueue<Runnable>();
+        pendingInvocations = new ConcurrentLinkedQueue<>();
     }
 
     /**
@@ -190,10 +199,12 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
      */
     protected AbstractSMTPTransport(int contextId) throws OXException {
         super();
+        smtpSessionRef = new AtomicReference<>();
+        cachedSmtpConfigRef = new AtomicReference<>();
         this.session = null;
         this.ctx = Services.getService(ContextService.class).getContext(contextId);
-        this.accountId = MailAccount.DEFAULT_ID;
-        pendingInvocations = new ConcurrentLinkedQueue<Runnable>();
+        this.accountId = Account.DEFAULT_ID;
+        pendingInvocations = new ConcurrentLinkedQueue<>();
     }
 
     /**
@@ -205,10 +216,12 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
      */
     protected AbstractSMTPTransport(Session session, int accountId) throws OXException {
         super();
+        smtpSessionRef = new AtomicReference<>();
+        cachedSmtpConfigRef = new AtomicReference<>();
         this.session = session;
         this.ctx = Services.getService(ContextService.class).getContext(session.getContextId());
         this.accountId = accountId;
-        pendingInvocations = new ConcurrentLinkedQueue<Runnable>();
+        pendingInvocations = new ConcurrentLinkedQueue<>();
     }
 
     // --------------------------------------------- Mandatory overrides ------------------------------------------------------------------
@@ -290,57 +303,57 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
     }
 
     protected void processAddressHeader(final MimeMessage mimeMessage) throws OXException, MessagingException {
-        {
+        { // NOSONARLINT
             final String str = mimeMessage.getHeader("From", null);
-            if (!com.openexchange.java.Strings.isEmpty(str)) {
+            if (!Strings.isEmpty(str)) {
                 final InternetAddress[] addresses = QuotedInternetAddress.parse(str, false);
                 checkRecipients(addresses);
                 mimeMessage.setFrom(addresses[0]);
             }
         }
-        {
+        { // NOSONARLINT
             final String str = mimeMessage.getHeader("Sender", null);
-            if (!com.openexchange.java.Strings.isEmpty(str)) {
+            if (!Strings.isEmpty(str)) {
                 final InternetAddress[] addresses = QuotedInternetAddress.parse(str, false);
                 checkRecipients(addresses);
                 mimeMessage.setSender(addresses[0]);
             }
         }
-        {
+        { // NOSONARLINT
             final String str = mimeMessage.getHeader("To", null);
-            if (!com.openexchange.java.Strings.isEmpty(str)) {
+            if (!Strings.isEmpty(str)) {
                 final InternetAddress[] addresses = QuotedInternetAddress.parse(str, false);
                 checkRecipients(addresses);
                 mimeMessage.setRecipients(RecipientType.TO, addresses);
             }
         }
-        {
+        { // NOSONARLINT
             final String str = mimeMessage.getHeader("Cc", null);
-            if (!com.openexchange.java.Strings.isEmpty(str)) {
+            if (!Strings.isEmpty(str)) {
                 final InternetAddress[] addresses = QuotedInternetAddress.parse(str, false);
                 checkRecipients(addresses);
                 mimeMessage.setRecipients(RecipientType.CC, addresses);
             }
         }
-        {
+        { // NOSONARLINT
             final String str = mimeMessage.getHeader("Bcc", null);
-            if (!com.openexchange.java.Strings.isEmpty(str)) {
+            if (!Strings.isEmpty(str)) {
                 final InternetAddress[] addresses = QuotedInternetAddress.parse(str, false);
                 checkRecipients(addresses);
                 mimeMessage.setRecipients(RecipientType.BCC, addresses);
             }
         }
-        {
+        { // NOSONARLINT
             final String str = mimeMessage.getHeader("Reply-To", null);
-            if (!com.openexchange.java.Strings.isEmpty(str)) {
+            if (!Strings.isEmpty(str)) {
                 final InternetAddress[] addresses = QuotedInternetAddress.parse(str, false);
                 checkRecipients(addresses);
                 mimeMessage.setReplyTo(addresses);
             }
         }
-        {
+        { // NOSONARLINT
             final String str = mimeMessage.getHeader("Disposition-Notification-To", null);
-            if (!com.openexchange.java.Strings.isEmpty(str)) {
+            if (!Strings.isEmpty(str)) {
                 if ("true".equalsIgnoreCase(str)) {
                     Address[] fromAddresses = mimeMessage.getFrom();
                     if (fromAddresses.length > 0 && fromAddresses[0] != null) {
@@ -355,7 +368,7 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
         }
     }
 
-    protected boolean checkRecipients(Address[] recipients) throws OXException {
+    protected static boolean checkRecipients(Address[] recipients) throws OXException {
         if ((recipients == null) || (recipients.length == 0)) {
             throw SMTPExceptionCode.MISSING_RECIPIENTS.create();
         }
@@ -379,7 +392,7 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
         if (MailProperties.getInstance().isSupportMsisdnAddresses()) {
             InternetAddress internetAddress;
             for (final Address address : recipients) {
-                if (MimeMessageUtility.POISON_ADDRESS == address) {
+                if (MimeMessageUtility.POISON_ADDRESS == address) { // NOSONARLINT
                     poisoned = Boolean.TRUE;
                 } else {
                     internetAddress = (InternetAddress) address;
@@ -406,8 +419,17 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
      * helper methods
      */
     protected javax.mail.Session getSMTPSession(SMTPConfig smtpConfig, boolean forceSecure) throws OXException {
+        if (smtpConfig.getAccount() != null && Constants.NAME_GUARD_GUEST.equals(smtpConfig.getAccount().getName())) {
+            // Do not use any SMTP settings for guard guest because SMTP is not used anyway
+            final Properties smtpProps = SMTPSessionProperties.getDefaultSessionProperties();
+            smtpProps.put("mail.smtp.starttls.required", "false");
+            return javax.mail.Session.getInstance(smtpProps, null);
+        }
+
+        javax.mail.Session smtpSession = smtpSessionRef.get();
         if (null == smtpSession) {
             synchronized (this) {
+                smtpSession = smtpSessionRef.get();
                 if (null == smtpSession) {
                     final Properties smtpProps = SMTPSessionProperties.getDefaultSessionProperties();
                     smtpProps.put("mail.smtp.class", JavaSMTPTransport.class.getName());
@@ -458,6 +480,21 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
                     } else {
                         smtpProps.put("mail.smtp.auth.mechanisms", "LOGIN PLAIN DIGEST-MD5 NTLM");
                     }
+                    /*
+                     * Allow XCLIENT?
+                     */
+                    if (smtpProperties.isAllowXClient()) {
+                        smtpProps.put("mail.smtp.allowxclient", "true");
+                    }
+                    /*
+                     * Blocked hosts
+                     */
+                    if (smtpConfig.getAccountId() != Account.DEFAULT_ID && !MailAccounts.isSecondaryAccount(smtpConfig.getAccountId(), smtpConfig.getSession().getUserId(), smtpConfig.getSession().getContextId())) {
+                        HostList blockedHosts = MailAccountUtils.getBlockedHosts();
+                        if (!blockedHosts.isEmpty()) {
+                            smtpProps.put("mail.smtp.blockedHosts", blockedHosts);
+                        }
+                    }
                     /*
                      * Check if a secure SMTP connection should be established
                      */
@@ -567,9 +604,9 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
                      */
                     // smtpProps.put(MIMESessionPropertyNames.PROP_SMTPHOST, smtpConfig.getServer());
                     // smtpProps.put(MIMESessionPropertyNames.PROP_SMTPPORT, sPort);
-                    javax.mail.Session smtpSession = javax.mail.Session.getInstance(smtpProps, null);
-                    smtpSession.addProvider(new Provider(Provider.Type.TRANSPORT, "smtp", JavaSMTPTransport.class.getName(), "OX Software GmbH", MailAccess.getVersion()));
-                    this.smtpSession = smtpSession;
+                    smtpSession = javax.mail.Session.getInstance(smtpProps, null);
+                    smtpSession.addProvider(new Provider(Provider.Type.TRANSPORT, "smtp", JavaSMTPTransport.class.getName(), "Open-Xchange GmbH", MailAccess.getVersion()));
+                    smtpSessionRef.set(smtpSession);
                 }
             }
         }
@@ -596,15 +633,19 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
 
     /**
      * Gets the connected SMTP transport.
+     * <p>
+     * <div style="margin-left: 0.1in; margin-right: 0.5in; margin-bottom: 0.1in; background-color:#FFDDDD;">
+     * <b>Don't forget to {@link SMTPTransport#close() close} the obtained SMTP transport.</b>
+     * </div>
      *
      * @return The connected SMTP transport
      * @throws OXException If connect attempt fails
      */
-    protected com.sun.mail.smtp.SMTPTransport getSmtpTransport() throws OXException {
+    public SMTPTransport getSmtpTransport() throws OXException {
         SMTPConfig smtpConfig = getTransportConfig();
         javax.mail.Session smtpSession = getSMTPSession(smtpConfig);
         try {
-            com.sun.mail.smtp.SMTPTransport transport = (com.sun.mail.smtp.SMTPTransport) smtpSession.getTransport(SMTP);
+            SMTPTransport transport = (SMTPTransport) smtpSession.getTransport(SMTP);
             connectTransport(transport, smtpConfig);
             return transport;
         } catch (NoSuchProviderException e) {
@@ -633,8 +674,9 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
      */
     protected void connectTransport(Transport transport, SMTPConfig smtpConfig, boolean forPing) throws OXException {
         // Check if account is disabled
+        SMTPConfig smtpConfigToUse = smtpConfig;
         if (false == forPing) {
-            Account account = getAccount(smtpConfig);
+            Account account = getAccount(smtpConfigToUse);
             if (null != account) {
                 if (account.isTransportDisabled()) {
                     if (account.isTransportOAuthAble() && account.getTransportOAuthId() >= 0) {
@@ -648,12 +690,15 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
                                 oAuthAccount = null;
                             }
                             if (null != oAuthAccount) {
-                                throw MailExceptionCode.MAIL_TRANSPORT_DISABLED_OAUTH.create(smtpConfig.getServer(), smtpConfig.getLogin(), I(session.getUserId()), I(session.getContextId()), oAuthAccount.getDisplayName());
+                                throw MailExceptionCode.MAIL_TRANSPORT_DISABLED_OAUTH.create(smtpConfigToUse.getServer(), smtpConfigToUse.getLogin(), I(session.getUserId()), I(session.getContextId()), oAuthAccount.getDisplayName());
                             }
                         }
                     }
 
-                    throw MailExceptionCode.MAIL_TRANSPORT_DISABLED.create(smtpConfig.getServer(), smtpConfig.getLogin(), I(session.getUserId()), I(session.getContextId()));
+                    throw MailExceptionCode.MAIL_TRANSPORT_DISABLED.create(smtpConfigToUse.getServer(), smtpConfigToUse.getLogin(), I(session.getUserId()), I(session.getContextId()));
+                }
+                if (account.isDeactivated()) {
+                    throw MailExceptionCode.MAIL_TRANSPORT_DEACTIVATED.create(smtpConfigToUse.getServer(), smtpConfigToUse.getLogin(), I(session.getUserId()), I(session.getContextId()));
                 }
                 if (TransportAuth.considerAsMailTransportAuth(account.getTransportAuth()) && (account instanceof MailAccount)) {
                     MailAccount mailAccount = (MailAccount) account;
@@ -663,44 +708,44 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
                             if (null != oauthService) {
                                 try {
                                     OAuthAccount oAuthAccount = oauthService.getAccount(session, mailAccount.getMailOAuthId());
-                                    throw MailExceptionCode.MAIL_TRANSPORT_DISABLED_OAUTH.create(smtpConfig.getServer(), smtpConfig.getLogin(), I(session.getUserId()), I(session.getContextId()), oAuthAccount.getDisplayName());
+                                    throw MailExceptionCode.MAIL_TRANSPORT_DISABLED_OAUTH.create(smtpConfigToUse.getServer(), smtpConfigToUse.getLogin(), I(session.getUserId()), I(session.getContextId()), oAuthAccount.getDisplayName());
                                 } catch (Exception x) {
                                     LOG.warn("Failed to load mail-associated OAuth account", x);
                                 }
                             }
                         }
 
-                        throw MailExceptionCode.MAIL_TRANSPORT_DISABLED.create(smtpConfig.getServer(), smtpConfig.getLogin(), I(session.getUserId()), I(session.getContextId()));
+                        throw MailExceptionCode.MAIL_TRANSPORT_DISABLED.create(smtpConfigToUse.getServer(), smtpConfigToUse.getLogin(), I(session.getUserId()), I(session.getContextId()));
                     }
                 }
             }
         }
 
-        final String server = IDNA.toASCII(smtpConfig.getServer());
-        final int port = smtpConfig.getPort();
-        final String login = smtpConfig.getLogin();
+        String server = IDNA.toASCII(smtpConfigToUse.getServer());
+        int port = smtpConfigToUse.getPort();
         try {
             if (smtpConfig.getSMTPProperties().isSmtpAuth()) {
-                final String password = smtpConfig.getPassword();
+                String login = smtpConfigToUse.getLogin();
+                String password = smtpConfigToUse.getPassword();
                 if (isKerberosAuth()) {
                     try {
-                        Subject.doAs(kerberosSubject, new SaslSmtpLoginAction(transport, server, port, login, password));
+                        smtpConfigToUse = (SMTPConfig) Subject.doAs(kerberosSubject, new SaslSmtpLoginAction(transport, server, port, login, password, this));
                     } catch (PrivilegedActionException e) {
                         handlePrivilegedActionException(e);
                     }
                 } else {
-                    doConnectTransport(transport, server, port, null == login ? "" : login, null == password ? "" : password, smtpConfig, session);
+                    smtpConfigToUse = doConnectTransport(transport, server, port, null == login ? "" : login, null == password ? "" : password, smtpConfigToUse, new HashMap<>(4));
                 }
             } else {
-                doConnectTransport(transport, server, port, null, null, smtpConfig, session);
+                smtpConfigToUse = doConnectTransport(transport, server, port, null , null, smtpConfigToUse, new HashMap<>(4));
             }
 
             Session session = this.session;
             if (session != null) {
                 AuditLogService auditLogService = Services.optService(AuditLogService.class);
                 if (null != auditLogService) {
-                    String eventId = forPing ? "smtp.external.login" : (MailAccount.DEFAULT_ID == accountId ? "smtp.primary.login" : "smtp.external.login");
-                    auditLogService.log(eventId, DefaultAttribute.valueFor(Name.LOGIN, session.getLoginName()), DefaultAttribute.valueFor(Name.IP_ADDRESS, session.getLocalIp()), DefaultAttribute.timestampFor(new Date()), DefaultAttribute.arbitraryFor("smtp.login", null == login ? "<none>" : login), DefaultAttribute.arbitraryFor("smtp.server", server), DefaultAttribute.arbitraryFor("smtp.port", Integer.toString(port)));
+                    String eventId = forPing ? "smtp.external.login" : (Account.DEFAULT_ID == accountId ? "smtp.primary.login" : "smtp.external.login");
+                    auditLogService.log(eventId, DefaultAttribute.valueFor(Name.LOGIN, session.getLoginName()), DefaultAttribute.valueFor(Name.IP_ADDRESS, session.getLocalIp()), DefaultAttribute.timestampFor(new Date()), DefaultAttribute.arbitraryFor("smtp.login", null == smtpConfigToUse.getLogin() ? "<none>" : smtpConfigToUse.getLogin()), DefaultAttribute.arbitraryFor("smtp.server", server), DefaultAttribute.arbitraryFor("smtp.port", Integer.toString(port)));
                 }
             }
         } catch (OXException e) {
@@ -710,7 +755,27 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
             }
             throw e;
         } catch (javax.mail.AuthenticationFailedException e) {
-            OXException oxe = MimeMailExceptionCode.TRANSPORT_INVALID_CREDENTIALS.create(e, smtpConfig.getServer(), e.getMessage());
+            if (AuthType.isOAuthType(smtpConfigToUse.getAuthType())) {
+                // Determine identifier of the associated OAuth account
+                int oauthAccountId = smtpConfigToUse.getOAuthAccountId();
+                if (oauthAccountId >= 0) {
+                    OAuthService oauthService = Services.optService(OAuthService.class);
+                    if (null == oauthService) {
+                        LOG.warn("Detected failed OAuth authentication, but unable to handle as needed service {} is missing", OAuthService.class.getSimpleName());
+                    } else {
+                        try {
+                            OAuthAccount oAuthAccount = oauthService.getAccount(session, oauthAccountId);
+                            API api = oAuthAccount.getAPI();
+                            Throwable cause = e.getCause();
+                            throw OAuthExceptionCodes.OAUTH_ACCESS_TOKEN_INVALID.create(cause, api.getDisplayName(), I(oAuthAccount.getId()), I(session.getUserId()), I(session.getContextId()));
+                        } catch (Exception x) {
+                            LOG.warn("Failed to handle failed OAuth authentication", x);
+                        }
+                    }
+                }
+            }
+
+            OXException oxe = MimeMailExceptionCode.TRANSPORT_INVALID_CREDENTIALS.create(e, smtpConfigToUse.getServer(), e.getMessage());
             if (accountId == Account.DEFAULT_ID) {
                 // Do log failed authentication attempts against primary mail server
                 LOG.warn("Failed authentication against primary transport server", oxe);
@@ -718,12 +783,15 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
             throw oxe;
         } catch (MessagingException e) {
             if (MimeMailException.isSSLHandshakeException(e)) {
-                List<Object> displayArgs = new ArrayList<>(2);
-                displayArgs.add(SSLExceptionCode.extractArgument(e, "fingerprint"));
-                displayArgs.add(server);
-                throw SSLExceptionCode.UNTRUSTED_CERTIFICATE.create(e.getCause(), displayArgs.toArray(new Object[] {}));
+                String fingerPrint = SSLExceptionCode.extractArgument(e, "fingerprint");
+                if (Strings.isNotEmpty(fingerPrint)) {
+                    List<Object> displayArgs = new ArrayList<>(2);
+                    displayArgs.add(fingerPrint);
+                    displayArgs.add(server);
+                    throw SSLExceptionCode.UNTRUSTED_CERTIFICATE.create(e.getCause(), displayArgs.toArray(new Object[] {}));
+                }
             }
-            throw handleMessagingException(e, smtpConfig);
+            throw handleMessagingException(e, smtpConfigToUse);
         }
     }
 
@@ -754,19 +822,27 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
      * @param port The port
      * @param user The user name
      * @param password The password to use
+     * @param state A map for storing/managing state
+     * @return The SMTP configuration actually used for authentication
      * @throws OXException If connect attempt fails
      * @throws MessagingException If connect attempt fails
      */
-    protected static void doConnectTransport(Transport transport, String host, int port, String user, String password, final SMTPConfig smtpConfig, final Session session) throws OXException, MessagingException {
+    protected SMTPConfig doConnectTransport(Transport transport, String host, int port, String user, String password, SMTPConfig smtpConfig, Map<String, Object> state) throws OXException, MessagingException {
+        boolean isPrimaryOrSecondaryAccount = smtpConfig != null && session != null && (smtpConfig.getAccountId() == Account.DEFAULT_ID || getAccount(smtpConfig).isSecondaryAccount());
+        if (isPrimaryOrSecondaryAccount) {
+            String remotePort = LogProperties.get(LogProperties.Name.GRIZZLY_REMOTE_PORT);
+            SMTPClientParameters.setDefaultClientParameters(transport, session, Strings.parsePositiveInt(remotePort));
+        }
         try {
             transport.connect(host, port, user, password);
+            return smtpConfig;
         } catch (javax.mail.AuthenticationFailedException e) {
             if (null == smtpConfig || null == session) {
                 throw e;
             }
 
-            boolean isPrimaryAccount = smtpConfig.getAccountId() == MailAccount.DEFAULT_ID;
-            if (false == isPrimaryAccount) {
+            if (false == isPrimaryOrSecondaryAccount) {
+                // Neither primary nor secondary account
                 AbstractTask<Void> task = new AbstractTask<Void>() {
 
                     @Override
@@ -785,16 +861,32 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
                 throw MailExceptionCode.AUTH_TYPE_NOT_SUPPORTED.create(smtpConfig.getAuthType().getName(), smtpConfig.getServer());
             }
 
-            if (isPrimaryAccount) {
+            if (isPrimaryOrSecondaryAccount) {
                 AuthenticationFailedHandlerService handlerService = Services.getService(AuthenticationFailedHandlerService.class);
                 if (handlerService != null) {
                     OXException oxe = MimeMailExceptionCode.TRANSPORT_INVALID_CREDENTIALS.create(e, smtpConfig.getServer(), e.getMessage());
-                    AuthenticationFailureHandlerResult result = handlerService.handleAuthenticationFailed(oxe, Service.TRANSPORT, smtpConfig, session);
+
+                    // Increment invocation count
+                    int currentInvocationCount = AuthenticationFailedStateUtility.getInvocationCount(state);
+                    if (currentInvocationCount == 0) {
+                        AuthenticationFailedStateUtility.setInvocationCount(1, state);
+                    } else {
+                        AuthenticationFailedStateUtility.setInvocationCount(currentInvocationCount + 1, state);
+                    }
+
+                    // Invoke authentication failed handlers
+                    AuthenticationFailureHandlerResult result = handlerService.handleAuthenticationFailed(oxe, Service.TRANSPORT, smtpConfig, session, state);
                     Type type = result.getType();
                     switch (type) {
                         case RETRY:
-                            transport.connect(host, port, user, password);
-                            return;
+                            cachedSmtpConfigRef.set(null);
+                            SMTPConfig newSmtpConfig = getTransportConfig();
+                            if (newSmtpConfig.getSMTPProperties().isSmtpAuth()) {
+                                String newUser = newSmtpConfig.getLogin();
+                                String newPassword = newSmtpConfig.getPassword();
+                                return doConnectTransport(transport, host, port, null == newUser ? "" : newUser, null == newPassword ? "" : newPassword, newSmtpConfig, state);
+                            }
+                            return doConnectTransport(transport, host, port, null, null, newSmtpConfig, state);
                         case EXCEPTION:
                             throw result.getError();
                         default:
@@ -803,26 +895,6 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
                 }
             }
 
-            if (AuthType.isOAuthType(smtpConfig.getAuthType())) {
-                // Determine identifier of the associated OAuth account
-                int oauthAccountId = smtpConfig.getOAuthAccountId();
-                if (oauthAccountId >= 0) {
-                    OAuthService oauthService = Services.optService(OAuthService.class);
-                    if (null == oauthService) {
-                        LOG.warn("Detected failed OAuth authentication, but unable to handle as needed service {} is missing", OAuthService.class.getSimpleName());
-                    } else {
-                        try {
-                            OAuthAccount oAuthAccount = oauthService.getAccount(session, oauthAccountId);
-                            API api = oAuthAccount.getAPI();
-                            Throwable cause = e.getCause();
-                            throw OAuthExceptionCodes.OAUTH_ACCESS_TOKEN_INVALID.create(cause, api.getDisplayName(), I(oAuthAccount.getId()), I(session.getUserId()), I(session.getContextId()));
-                        } catch (Exception x) {
-                            LOG.warn("Failed to handle failed OAuth authentication", x);
-                        }
-                    }
-                }
-            }
-
             throw e;
         }
     }
@@ -1023,11 +1095,11 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
         transport.sendMessage(messageToSend, recipients);
     }
 
-    private void closeSafe(Transport transport) {
+    private static void closeSafe(Transport transport) {
         if (null != transport) {
             try {
                 transport.close();
-            } catch (MessagingException e) {
+            } catch (Exception e) {
                 if (ExceptionUtils.isEitherOf(e, java.net.SocketException.class)) {
                     // Connection already closed by remote host
                 } else {
@@ -1043,13 +1115,13 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
 
     @Override
     public SMTPConfig getTransportConfig() throws OXException {
-        SMTPConfig tmp = cachedSmtpConfig;
+        SMTPConfig tmp = cachedSmtpConfigRef.get();
         if (tmp == null) {
             synchronized (this) {
-                tmp = cachedSmtpConfig;
+                tmp = cachedSmtpConfigRef.get();
                 if (tmp == null) {
                     tmp = createSMTPConfig();
-                    cachedSmtpConfig = tmp;
+                    cachedSmtpConfigRef.set(tmp);
                 }
             }
         }
@@ -1377,7 +1449,7 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
      * @return <code>true</code> for Kerberos authentication; otherwise <code>false</code>
      */
     private boolean isKerberosAuth() {
-        return MailAccount.DEFAULT_ID == accountId && null != kerberosSubject;
+        return Account.DEFAULT_ID == accountId && null != kerberosSubject;
     }
 
     private long getMaxMailSize() {
@@ -1425,7 +1497,7 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
 
     private void checkMimeVersionHeader(final MimeMessage mimeMessage) throws MessagingException, IOException {
         final String header = mimeMessage.getHeader("Content-Type", null);
-        if (null != header && header.toLowerCase().startsWith("multipart/")) {
+        if (null != header && Strings.asciiLowerCase(header).startsWith("multipart/")) {
             final Multipart multipart = (Multipart) mimeMessage.getContent();
             final int count = multipart.getCount();
             for (int i = 0; i < count; i++) {
@@ -1437,7 +1509,7 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
     private void checkMimeVersionHeader(final Part part) throws MessagingException, IOException {
         final String[] header = part.getHeader("Content-Type");
         if (null != header && header.length > 0 && null != header[0]) {
-            final String cts = header[0].toLowerCase();
+            final String cts = Strings.asciiLowerCase(header[0]);
             if (cts.startsWith("multipart/")) {
                 final Multipart multipart = (Multipart) part.getContent();
                 final int count = multipart.getCount();
@@ -1464,20 +1536,20 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
         }
     }
 
-    private String getFallbackHostname() {
+    private static String getFallbackHostname() {
         final String serverName = LogProperties.getLogProperty(LogProperties.Name.GRIZZLY_SERVER_NAME);
         return null == serverName ? getStaticHostName() : serverName;
     }
 
     private static String getStaticHostName() {
-        final UnknownHostException warning = warnSpam;
+        final UnknownHostException warning = WARN_SPAM_REF.get();
         if (warning != null) {
             LOG.error("Can't resolve my own hostname, using 'localhost' instead, which is certainly not what you want!", warning);
         }
-        return staticHostName;
+        return STATIC_HOST_NAME_REF.get();
     }
 
-    private static final boolean isPoisoned(final Address[] recipients) {
+    private static boolean isPoisoned(final Address[] recipients) {
         if ((recipients == null) || (recipients.length == 0)) {
             return false;
         }
@@ -1564,7 +1636,7 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
         }
 
         private void fillAddressesFromEvent(TransportEvent e) {
-            javax.mail.Address[] arr = e.getInvalidAddresses();
+            Address[] arr = e.getInvalidAddresses();
             if (null != arr) {
                 statusInfo.getInvalidAddresses().addAll(Arrays.asList(arr));
             }
@@ -1618,10 +1690,10 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
         }
 
         private void logAddressesFromEvent(TransportEvent e) {
-            javax.mail.Address[] arr = e.getInvalidAddresses();
+            Address[] arr = e.getInvalidAddresses();
             if (null != arr) {
                 for (Address address : arr) {
-                    String eventId = MailAccount.DEFAULT_ID == accountId ? "smtp.primary.invalid" : "smtp.external.invalid";
+                    String eventId = Account.DEFAULT_ID == accountId ? "smtp.primary.invalid" : "smtp.external.invalid";
                     auditLogService.log(eventId, DefaultAttribute.valueFor(Name.LOGIN, session.getLoginName()), DefaultAttribute.valueFor(Name.IP_ADDRESS, session.getLocalIp()), DefaultAttribute.timestampFor(new Date()), DefaultAttribute.arbitraryFor("smtp.login", null == login ? "<none>" : login), DefaultAttribute.arbitraryFor("smtp.server", server), DefaultAttribute.arbitraryFor("smtp.port", Integer.toString(port)), DefaultAttribute.arbitraryFor("smtp.invalid", address instanceof InternetAddress ? ((InternetAddress) address).getAddress() : address.toString()));
                 }
             }
@@ -1629,7 +1701,7 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
             arr = e.getValidUnsentAddresses();
             if (null != arr) {
                 for (Address address : arr) {
-                    String eventId = MailAccount.DEFAULT_ID == accountId ? "smtp.primary.unsent" : "smtp.external.unsent";
+                    String eventId = Account.DEFAULT_ID == accountId ? "smtp.primary.unsent" : "smtp.external.unsent";
                     auditLogService.log(eventId, DefaultAttribute.valueFor(Name.LOGIN, session.getLoginName()), DefaultAttribute.valueFor(Name.IP_ADDRESS, session.getLocalIp()), DefaultAttribute.timestampFor(new Date()), DefaultAttribute.arbitraryFor("smtp.login", null == login ? "<none>" : login), DefaultAttribute.arbitraryFor("smtp.server", server), DefaultAttribute.arbitraryFor("smtp.port", Integer.toString(port)), DefaultAttribute.arbitraryFor("smtp.unsent", address instanceof InternetAddress ? ((InternetAddress) address).getAddress() : address.toString()));
                 }
             }
@@ -1637,7 +1709,7 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
             arr = e.getValidSentAddresses();
             if (null != arr) {
                 for (Address address : arr) {
-                    String eventId = MailAccount.DEFAULT_ID == accountId ? "smtp.primary.sent" : "smtp.external.sent";
+                    String eventId = Account.DEFAULT_ID == accountId ? "smtp.primary.sent" : "smtp.external.sent";
                     auditLogService.log(eventId, DefaultAttribute.valueFor(Name.LOGIN, session.getLoginName()), DefaultAttribute.valueFor(Name.IP_ADDRESS, session.getLocalIp()), DefaultAttribute.timestampFor(new Date()), DefaultAttribute.arbitraryFor("smtp.login", null == login ? "<none>" : login), DefaultAttribute.arbitraryFor("smtp.server", server), DefaultAttribute.arbitraryFor("smtp.port", Integer.toString(port)), DefaultAttribute.arbitraryFor("smtp.sent", address instanceof InternetAddress ? ((InternetAddress) address).getAddress() : address.toString()));
                 }
             }
@@ -1669,20 +1741,21 @@ abstract class AbstractSMTPTransport extends MailTransport implements MimeSuppor
         private final int port;
         private final String login;
         private final String pw;
+        private final AbstractSMTPTransport smtpTransport;
 
-        SaslSmtpLoginAction(Transport transport, String server, int port, String login, String pw) {
+        SaslSmtpLoginAction(Transport transport, String server, int port, String login, String pw, AbstractSMTPTransport smtpTransport) {
             super();
             this.transport = transport;
             this.server = server;
             this.port = port;
             this.login = login;
             this.pw = pw;
+            this.smtpTransport = smtpTransport;
         }
 
         @Override
         public Object run() throws Exception {
-            doConnectTransport(transport, server, port, login, pw, null, null);
-            return null;
+            return smtpTransport.doConnectTransport(transport, server, port, login, pw, null, new HashMap<String, Object>(4));
         }
     } // End of class SaslSmtpLoginAction
 
