@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -21,6 +21,7 @@
 
 package com.openexchange.smtp;
 
+import static com.openexchange.java.Autoboxing.I;
 import static com.openexchange.mail.mime.utils.MimeMessageUtility.parseAddressList;
 import static com.openexchange.mail.text.TextProcessing.performLineFolding;
 import java.text.DateFormat;
@@ -30,6 +31,7 @@ import javax.mail.Address;
 import javax.mail.Message.RecipientType;
 import javax.mail.MessagingException;
 import javax.mail.Multipart;
+import javax.mail.internet.AddressException;
 import javax.mail.internet.InternetAddress;
 import javax.mail.internet.MailDateFormat;
 import javax.mail.internet.MimeBodyPart;
@@ -40,6 +42,7 @@ import com.openexchange.exception.OXException;
 import com.openexchange.groupware.i18n.MailStrings;
 import com.openexchange.groupware.ldap.UserStorage;
 import com.openexchange.i18n.tools.StringHelper;
+import com.openexchange.java.Strings;
 import com.openexchange.log.LogProperties;
 import com.openexchange.mail.MailPath;
 import com.openexchange.mail.api.MailAccess;
@@ -49,6 +52,7 @@ import com.openexchange.mail.dataobjects.MailMessage;
 import com.openexchange.mail.mime.ContentType;
 import com.openexchange.mail.mime.MessageHeaders;
 import com.openexchange.mail.mime.MimeMailException;
+import com.openexchange.mail.mime.QuotedInternetAddress;
 import com.openexchange.mail.mime.filler.MimeMessageFiller;
 import com.openexchange.mail.mime.utils.MimeMessageUtility;
 import com.openexchange.mail.transport.config.ITransportProperties;
@@ -125,7 +129,7 @@ public final class DefaultSMTPTransport extends AbstractSMTPTransport {
         try {
             access = MailAccess.getInstance(session, msgref.getAccountId());
             access.connect();
-            MimeMessageFiller.setReplyHeaders(access.getMessageStorage().getMessage(msgref.getFolder(), msgref.getMailID(), false), mimeMessage);
+            MimeMessageFiller.setReplyHeaders(access.getMessageStorage().getMessage(msgref.getFolder(), msgref.getMailID(), false), mimeMessage, I(MailProperties.getInstance().getMaxLengthForReferencesHeader(session.getUserId(), session.getContextId())));
         } finally {
             if (null != access) {
                 access.close(true);
@@ -135,7 +139,8 @@ public final class DefaultSMTPTransport extends AbstractSMTPTransport {
 
     @Override
     protected SMTPMessageFiller createSMTPMessageFiller(UserSettingMail optMailSettings) throws OXException {
-        return new SMTPMessageFiller(getTransportConfig().getSMTPProperties(), session, ctx, null == optMailSettings ? usm : optMailSettings);
+        int remotePort = Strings.parsePositiveInt(LogProperties.get(LogProperties.Name.GRIZZLY_REMOTE_PORT));
+        return new SMTPMessageFiller(getTransportConfig().getSMTPProperties(), session, remotePort, ctx, null == optMailSettings ? usm : optMailSettings);
     }
 
     @Override
@@ -178,20 +183,32 @@ public final class DefaultSMTPTransport extends AbstractSMTPTransport {
                 }
             }
             final SMTPMessage smtpMessage = new SMTPMessage(getSMTPSession());
-            final String userMail = UserStorage.getInstance().getUser(session.getUserId(), ctx).getMail();
             /*
              * Set from
              */
-            final String from;
+            InternetAddress from;
             if (fromAddr == null) {
-                if ((usm.getSendAddr() == null) && (userMail == null)) {
-                    throw SMTPExceptionCode.NO_SEND_ADDRESS_FOUND.create();
+                String sendAddress = usm.getSendAddr();
+                if (sendAddress == null) {
+                    sendAddress = UserStorage.getInstance().getUser(session.getUserId(), ctx).getMail();
+                    if (sendAddress == null) {
+                        throw SMTPExceptionCode.NO_SEND_ADDRESS_FOUND.create();
+                    }
                 }
-                from = usm.getSendAddr() == null ? userMail : usm.getSendAddr();
+                InternetAddress[] addressList = parseAddressList(sendAddress, false);
+                from = addressList[0];
+                smtpMessage.addFrom(addressList);
             } else {
-                from = fromAddr;
+                try {
+                    InternetAddress[] addrs = QuotedInternetAddress.parse(fromAddr, false);
+                    smtpMessage.addFrom(addrs);
+                    from = addrs[0];
+                } catch (AddressException e) {
+                    InternetAddress addr = MimeMessageUtility.parseCraftedAddress(fromAddr);
+                    smtpMessage.addFrom(new Address[] { addr });
+                    from = addr;
+                }
             }
-            smtpMessage.addFrom(parseAddressList(from, false));
             /*
              * Set to
              */
@@ -224,7 +241,8 @@ public final class DefaultSMTPTransport extends AbstractSMTPTransport {
              * Set common headers
              */
             smtpConfig = getTransportConfig();
-            new SMTPMessageFiller(smtpConfig.getSMTPProperties(), session, ctx, usm).setAccountId(accountId).setCommonHeaders(smtpMessage);
+            int remotePort = Strings.parsePositiveInt(LogProperties.get(LogProperties.Name.GRIZZLY_REMOTE_PORT));
+            new SMTPMessageFiller(smtpConfig.getSMTPProperties(), session, remotePort, ctx, usm).setAccountId(accountId).setCommonHeaders(smtpMessage);
             /*
              * Compose body
              */
@@ -240,7 +258,7 @@ public final class DefaultSMTPTransport extends AbstractSMTPTransport {
                 final String txt = performLineFolding(
                     strHelper.getString(MailStrings.ACK_NOTIFICATION_TEXT)
                     .replaceFirst("#DATE#", sentDate == null ? "" : quoteReplacement(DateFormat.getDateInstance(DateFormat.LONG, locale).format(sentDate)))
-                    .replaceFirst("#RECIPIENT#", quoteReplacement(from)).replaceFirst("#SUBJECT#", quoteReplacement(srcMail.getSubject())), usm.getAutoLinebreak());
+                    .replaceFirst("#RECIPIENT#", quoteReplacement(craftDisplayStringFrom(from))).replaceFirst("#SUBJECT#", quoteReplacement(srcMail.getSubject())), usm.getAutoLinebreak());
                 MessageUtility.setText(txt, defaultMimeCS, text);
                 // text.setText(txt,defaultMimeCS);
                 text.setHeader(MessageHeaders.HDR_MIME_VERSION, "1.0");
@@ -254,7 +272,7 @@ public final class DefaultSMTPTransport extends AbstractSMTPTransport {
             {
                 final MimeBodyPart ack = new MimeBodyPart();
                 final String msgId = srcMail.getFirstHeader(MessageHeaders.HDR_MESSAGE_ID);
-                final String txt = strHelper.getString(ACK_TEXT).replaceFirst("#FROM#", quoteReplacement(from)).replaceFirst(
+                final String txt = strHelper.getString(ACK_TEXT).replaceFirst("#FROM#", quoteReplacement(from.toUnicodeString())).replaceFirst(
                     "#MSG ID#",
                     quoteReplacement(msgId));
                 MessageUtility.setText(txt, defaultMimeCS, ack);
@@ -278,6 +296,14 @@ public final class DefaultSMTPTransport extends AbstractSMTPTransport {
         }
     }
 
+    private static String craftDisplayStringFrom(InternetAddress address) {
+        String personal = address.getPersonal();
+        if (Strings.isEmpty(personal)) {
+            return address.getAddress();
+        }
+        return new StringBuilder(personal).append(" <").append(address.getAddress()).append('>').toString();
+    }
+
     @Override
     protected ITransportProperties createNewMailProperties() throws OXException {
         MailAccountStorageService storageService = Services.getService(MailAccountStorageService.class);
