@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -192,6 +192,11 @@ public class NoReplySMTPTransport extends AbstractSMTPTransport {
             return smtpProperties.isSendPartial();
         }
 
+        @Override
+        public boolean isAllowXClient() {
+            return smtpProperties.isAllowXClient();
+        }
+
         @Override
         public int getSmtpTimeout() {
             return smtpProperties.getSmtpTimeout();
