@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -44,6 +44,7 @@ import com.openexchange.java.Streams;
 import com.openexchange.java.Strings;
 import com.openexchange.mail.mime.MimeDefaultSession;
 import com.openexchange.net.ssl.SSLSocketFactoryProvider;
+import com.openexchange.net.ssl.config.SSLConfigurationService;
 import com.openexchange.smtp.config.ISMTPProperties;
 import com.openexchange.smtp.services.Services;
 import com.sun.mail.util.SocketFetcher;
@@ -315,6 +316,11 @@ public final class SMTPCapabilityCache {
             String sslProtocols = smtpProperties.getSSLProtocols();
             if (Strings.isNotEmpty(sslProtocols)) {
                 imapprops.put("mail.smtp.ssl.protocols", sslProtocols);
+            } else {
+                SSLConfigurationService sslConfigService = Services.getService(SSLConfigurationService.class);
+                if (sslConfigService != null) {
+                    imapprops.put("mail.smtp.ssl.protocols", Strings.toWhitespaceSeparatedList(sslConfigService.getSupportedProtocols()));
+                }
             }
         }
 
@@ -322,6 +328,11 @@ public final class SMTPCapabilityCache {
             String sslCipherSuites = smtpProperties.getSSLCipherSuites();
             if (Strings.isNotEmpty(sslCipherSuites)) {
                 imapprops.put("mail.smtp.ssl.ciphersuites", sslCipherSuites);
+            } else {
+                SSLConfigurationService sslConfigService = Services.getService(SSLConfigurationService.class);
+                if (sslConfigService != null) {
+                    imapprops.put("mail.smtp.ssl.ciphersuites", Strings.toWhitespaceSeparatedList(sslConfigService.getSupportedCipherSuites()));
+                }
             }
         }
     }
