@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -30,55 +30,55 @@ public interface ISMTPProperties extends ITransportProperties {
      *
      * @return the smtpLocalhost
      */
-    public String getSmtpLocalhost();
+    String getSmtpLocalhost();
 
     /**
      * Gets the smtpAuth
      *
      * @return the smtpAuth
      */
-    public boolean isSmtpAuth();
+    boolean isSmtpAuth();
 
     /**
      * Gets the smtpEnvelopeFrom
      *
      * @return the smtpEnvelopeFrom
      */
-    public boolean isSmtpEnvelopeFrom();
+    boolean isSmtpEnvelopeFrom();
 
     /**
      * Gets the logTransport flag
      *
      * @return the logTransport flag
      */
-    public boolean isLogTransport();
+    boolean isLogTransport();
 
     /**
      * Gets the smtpAuthEnc
      *
      * @return the smtpAuthEnc
      */
-    public String getSmtpAuthEnc();
+    String getSmtpAuthEnc();
 
     /**
      * Gets the smtpTimeout
      *
      * @return the smtpTimeout
      */
-    public int getSmtpTimeout();
+    int getSmtpTimeout();
 
     /**
      * Gets the smtpConnectionTimeout
      *
      * @return the smtpConnectionTimeout
      */
-    public int getSmtpConnectionTimeout();
+    int getSmtpConnectionTimeout();
 
     /**
      * Gets supported SSL protocols
      * @return Supported SSL protocols
      */
-    public String getSSLProtocols();
+    String getSSLProtocols();
 
     /**
      * Gets the SSL cipher suites that will be enabled for SSL connections. The property value is a whitespace separated list of tokens
@@ -86,20 +86,30 @@ public interface ISMTPProperties extends ITransportProperties {
      *
      * @return The SSL cipher suites
      */
-    public String getSSLCipherSuites();
+    String getSSLCipherSuites();
 
     /**
      * Whether partial send is allowed or message transport is supposed to be aborted.
      *
      * @return <code>true</code> if partial send is allowed; otherwise <code>false</code>
      */
-    public boolean isSendPartial();
-    
+    boolean isSendPartial();
+
+    /**
+     * Whether XCLIENT extension should be used if supported by SMTP server.
+     * <p>
+     * When an SMTP server announces support for the XCLIENT command, an SMTP client may send information that overrides one or more
+     * client-related session attributes.
+     *
+     * @return <code>true</code> if XCLIENT is allowed; otherwise <code>false</code>
+     */
+    boolean isAllowXClient();
+
     /**
      * Get the primary address header to append.
-     * 
+     *
      * @return The primary address header name or <code>null</code> if not set
      */
-    public String getPrimaryAddressHeader();
+    String getPrimaryAddressHeader();
 
 }
