@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -198,6 +198,23 @@ public final class MailAccountSMTPProperties extends MailAccountTransportPropert
         return lookUpBoolProperty("com.openexchange.smtp.sendPartial", SMTPProperties.getInstance().isSendPartial());
     }
 
+    @Override
+    public boolean isAllowXClient() {
+        String allowXClientStr = getAccountProperty("com.openexchange.smtp.allowXCLIENT");
+        if (null != allowXClientStr) {
+            return Boolean.parseBoolean(allowXClientStr.trim());
+        }
+
+        if (mailAccountId == PRIMARY) {
+            allowXClientStr = lookUpProperty("com.openexchange.smtp.primary.allowXCLIENT");
+            if (null != allowXClientStr) {
+                return Boolean.parseBoolean(allowXClientStr.trim());
+            }
+        }
+
+        return lookUpBoolProperty("com.openexchange.smtp.allowXCLIENT", SMTPProperties.getInstance().isAllowXClient());
+    }
+
     @Override
     public boolean isSmtpEnvelopeFrom() {
         if (mailAccountId != PRIMARY) {
