@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -21,12 +21,15 @@
 
 package com.openexchange.smtp.config;
 
+import static com.openexchange.java.Autoboxing.I;
+import com.openexchange.crypto.CryptoErrorMessage;
 import java.net.InetAddress;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.net.UnknownHostException;
 import javax.mail.internet.idn.IDNA;
 import com.openexchange.exception.OXException;
+import com.openexchange.java.InetAddresses;
 import com.openexchange.mail.MailExceptionCode;
 import com.openexchange.mail.api.AuthType;
 import com.openexchange.mail.api.MailCapabilities;
@@ -143,7 +146,7 @@ public final class SMTPConfig extends TransportConfig implements TransportAuthSu
     public InetAddress getSmtpServerAddress() throws OXException {
         if (null == smtpServerAddress) {
             try {
-                smtpServerAddress = InetAddress.getByName(getServer());
+                smtpServerAddress = InetAddresses.forString(getServer());
             } catch (UnknownHostException e) {
                 throw SMTPExceptionCode.IO_ERROR.create(e, e.getMessage());
             }
@@ -200,7 +203,15 @@ public final class SMTPConfig extends TransportConfig implements TransportAuthSu
                     authType = AuthType.parse(tokenInfo.getAuthMechanism());
                 } else {
                     login = saneLogin(account.getTransportLogin());
-                    password = MailPasswordUtil.decrypt(account.getTransportPassword(), session, account.getId(), login, account.getTransportServer());
+                    try {
+                        password = MailPasswordUtil.decrypt(account.getTransportPassword(), session, account.getId(), login, account.getTransportServer());
+                    } catch (OXException e) {
+                        if (CryptoErrorMessage.BadPassword.equals(e)) {
+                            // Account's password could not be decrypted
+                            throw MailExceptionCode.CANNOT_DECRYPT_PASSWORD.create(e, account.getTransportServer(), login, I(session.getUserId()), I(session.getContextId()));
+                        }
+                        throw e;
+                    }
                 }
                 break;
             case NONE:
