@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -68,6 +68,8 @@ public final class SMTPProperties extends AbstractProtocolProperties implements
 
     private boolean sendPartial;
 
+    private boolean allowXClient;
+
     private String smtpAuthEnc;
 
     private int smtpTimeout;
@@ -125,6 +127,14 @@ public final class SMTPProperties extends AbstractProtocolProperties implements
             args.add(lineSeparator);
         }
 
+        {
+            String allowXClientStr = configuration.getProperty("com.openexchange.smtp.allowXCLIENT", "false").trim();
+            allowXClient = Boolean.parseBoolean(allowXClientStr);
+            logBuilder.append("    Allow XCLIENT: {}{}");
+            args.add(B(allowXClient));
+            args.add(lineSeparator);
+        }
+
         {
             final String smtpEnvFromStr = configuration.getProperty("com.openexchange.smtp.setSMTPEnvelopeFrom", "false").trim();
             smtpEnvelopeFrom = Boolean.parseBoolean(smtpEnvFromStr);
@@ -145,7 +155,7 @@ public final class SMTPProperties extends AbstractProtocolProperties implements
             final String smtpAuthEncStr = configuration.getProperty("com.openexchange.smtp.smtpAuthEnc", "UTF-8").trim();
             if (CharsetDetector.isValid(smtpAuthEncStr)) {
                 smtpAuthEnc = smtpAuthEncStr;
-                logBuilder.append("    SMTP Auth Encoding: ").append(smtpAuthEnc).append('\n');
+                logBuilder.append("    SMTP Auth Encoding: {}{}");
                 args.add(smtpAuthEnc);
                 args.add(lineSeparator);
             } else {
@@ -191,10 +201,13 @@ public final class SMTPProperties extends AbstractProtocolProperties implements
             }
         }
 
-        sslProtocols = configuration.getProperty("com.openexchange.smtp.ssl.protocols", "SSLv3 TLSv1").trim();
-        logBuilder.append("    Supported SSL protocols: {}{}");
-        args.add(sslProtocols);
-        args.add(lineSeparator);
+        {
+            final String tmp = configuration.getProperty("com.openexchange.smtp.ssl.protocols", "").trim();
+            this.sslProtocols = Strings.isEmpty(tmp) ? null : tmp;
+            logBuilder.append("    Supported SSL protocols: {}{}");
+            args.add(null == this.sslProtocols ? "<default>" : sslProtocols);
+            args.add(lineSeparator);
+        }
 
         {
             final String tmp = configuration.getProperty("com.openexchange.smtp.ssl.ciphersuites", "").trim();
@@ -220,13 +233,14 @@ public final class SMTPProperties extends AbstractProtocolProperties implements
     protected void resetFields() {
         smtpLocalhost = null;
         sendPartial = false;
+        allowXClient = false;
         smtpAuth = false;
         smtpEnvelopeFrom = false;
         smtpAuthEnc = null;
         smtpTimeout = 0;
         smtpConnectionTimeout = 0;
         logTransport = false;
-        sslProtocols = "SSLv3 TLSv1";
+        sslProtocols = null;
         cipherSuites = null;
         primaryAdressHeader = null;
     }
@@ -246,6 +260,11 @@ public final class SMTPProperties extends AbstractProtocolProperties implements
         return sendPartial;
     }
 
+    @Override
+    public boolean isAllowXClient() {
+        return allowXClient;
+    }
+
     @Override
     public boolean isSmtpEnvelopeFrom() {
         return smtpEnvelopeFrom;
