@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -60,11 +60,12 @@ public final class SMTPMessageFiller extends MimeMessageFiller {
      *
      * @param smtpProperties
      * @param session The session
+     * @param remotePort The client's remote port number or <code>-1</code>
      * @param ctx The context
      * @param usm The user's mail settings
      */
-    public SMTPMessageFiller(ISMTPProperties smtpProperties, Session session, Context ctx, UserSettingMail usm) {
-        super(new SessionCompositionParameters(session, ctx, usm));
+    public SMTPMessageFiller(ISMTPProperties smtpProperties, Session session, int remotePort, Context ctx, UserSettingMail usm) {
+        super(new SessionCompositionParameters(session, remotePort, ctx, usm));
         this.smtpProperties = smtpProperties;
     }
 
